definition module Control.Monad.Maybe

from Control.Applicative import class pure, class <*>, class Applicative, class Alternative
from Control.Monad import class Monad
from Control.Monad.Fail import class MonadFail
from Control.Monad.Trans import class MonadTrans
from Data.Functor import class Functor

/**
 * The Maybe monad transformer.
 */
:: MaybeT m a = MaybeT !(m (? a))

/**
 * Runs a MaybeT as the monad wrapped inside the transformer.
 */
runMaybeT :: !(MaybeT m a) -> m (? a)

/**
 * Transforms the computation inside a transformer.
 *
 * @param The computation transformation.
 * @param The transformer to be transformed.
 * @result The transformed transformer.
 */
mapMaybeT :: !((m (? a)) -> n (? b)) !(MaybeT m a) -> MaybeT n b

instance Functor (MaybeT m) | Functor m
instance pure (MaybeT m) | pure m
instance <*> (MaybeT m) | Monad m
instance Alternative (MaybeT m) | Monad m
instance Monad (MaybeT m) | Monad m
instance MonadFail (MaybeT m) | MonadFail m
instance MonadTrans MaybeT
where
	liftT :: !(a b) -> MaybeT a b | Monad a
