implementation module Control.Monad.Maybe

import Control.Applicative
import Control.Monad
import Control.Monad.Fail
import Control.Monad.Trans
import Data.Func
import Data.Functor

runMaybeT :: !(MaybeT m a) -> m (? a)
runMaybeT (MaybeT f) = f

mapMaybeT :: !((m (? a)) -> n (? b)) !(MaybeT m a) -> MaybeT n b
mapMaybeT f m = MaybeT $ f $ runMaybeT m

instance Functor (MaybeT m) | Functor m where
	fmap f m = mapMaybeT (fmap $ fmap f) m

instance pure (MaybeT m) | pure m where pure x = MaybeT (pure (?Just x))

instance <*> (MaybeT m) | Monad m where
	(<*>) mf mx = MaybeT $
		runMaybeT mf >>= \mb_f ->
		case mb_f of
			?None = pure ?None
			?Just f  =
				runMaybeT mx >>= \mb_x ->
				case mb_x of
					?None   = pure ?None
					?Just x = pure $ ?Just $ f x

instance Alternative (MaybeT m) | Monad m where
	empty = MaybeT $ pure ?None

	(<|>) x y = MaybeT $
		runMaybeT x >>= \v ->
		case v of
			?None    -> runMaybeT y
			?Just _  -> pure v

instance Monad (MaybeT m) | Monad m where
	bind x f = MaybeT $
		runMaybeT x >>= \v ->
		case v of
			?None   -> pure ?None
			?Just y -> runMaybeT $ f y

instance MonadFail (MaybeT m) | MonadFail m where
	fail s = liftT (fail s)

instance MonadTrans MaybeT
where
	liftT :: !(a b) -> MaybeT a b | Monad a
	liftT m = MaybeT $ ?Just <$> m
