definition module Data.Monoid

from StdOverloaded import class +, class *, class zero, class one
import qualified StdList

class Semigroup a
where
	mappend :: !a a -> a

	(<++>) infixr 6 :: a a -> a | Semigroup a
	(<++>) ma mb :== mappend ma mb

class Monoid a | Semigroup a
where
	mempty :: a

	mconcat :: !.[a] -> a | Monoid a
	mconcat xs :== mconcat xs
	where
		mconcat []    = mempty
		mconcat [a:x] = mconcat a (mconcat x)

instance Semigroup ()
instance Semigroup (x, y) | Semigroup x & Semigroup y where mappend :: !(x, y) !(x, y) -> (x, y) | Semigroup x & Semigroup y
instance Semigroup (x, y, z) | Semigroup x & Semigroup y & Semigroup z where mappend :: !(x, y, z) !(x, y, z) -> (x, y, z) | Semigroup x & Semigroup y & Semigroup z
instance Semigroup (x, y, z, a) | Semigroup x & Semigroup y & Semigroup z & Semigroup a where mappend :: !(x, y, z, a) !(x, y, z, a) -> (x, y, z, a) | Semigroup x & Semigroup y & Semigroup z & Semigroup a
instance Semigroup (x, y, z, a, b) | Semigroup x & Semigroup y & Semigroup z & Semigroup a & Semigroup b where mappend :: !(x, y, z, a, b) !(x, y, z, a, b) -> (x, y, z, a, b) | Semigroup x & Semigroup y & Semigroup z & Semigroup a & Semigroup b
instance Semigroup (x, y, z, a, b, c) | Semigroup x & Semigroup y & Semigroup z & Semigroup a & Semigroup b & Semigroup c where mappend :: !(x, y, z, a, b, c) !(x, y, z, a, b, c) -> (x, y, z, a, b, c) | Semigroup x & Semigroup y & Semigroup z & Semigroup a & Semigroup b & Semigroup c
instance Semigroup (? a) | Semigroup a where mappend :: !(? a) !(? a) -> ? a | Semigroup a
instance Semigroup (?^ a) | Semigroup a where mappend :: !(?^ a) !(?^ a) -> ?^ a | Semigroup a
instance Semigroup [a], [!a], [a!], [!a!]
instance Semigroup (a -> b) | Semigroup b

instance Monoid ()
instance Monoid (x, y) | Monoid x & Monoid y
instance Monoid (x, y, z) | Monoid x & Monoid y & Monoid z
instance Monoid (x, y, z, a) | Monoid x & Monoid y & Monoid z & Monoid a
instance Monoid (x, y, z, a, b) | Monoid x & Monoid y & Monoid z & Monoid a & Monoid b
instance Monoid (x, y, z, a, b, c) | Monoid x & Monoid y & Monoid z & Monoid a & Monoid b & Monoid c
instance Monoid [a], [!a], [a!], [!a!]
instance Monoid (?a), (?^a)
instance Monoid (a -> b) | Monoid b

:: Dual a = Dual a

:: Endo a = Endo (a -> a)

:: All = All Bool

:: Any = Any Bool

:: Sum a = Sum a

:: Product a = Product a

:: First a = First (?a)

:: Last a = Last (?a)

instance Semigroup (Dual a) | Semigroup a
where
	mappend :: !(Dual a) !(Dual a) -> Dual a | Semigroup a

instance Semigroup (Endo .a)
where
	mappend :: !(Endo .a) !(Endo .a) -> Endo .a

instance Semigroup All
where
	mappend :: !All !All -> All

instance Semigroup Any
where
	mappend :: !Any !Any -> Any

instance Semigroup (Sum a) | +, zero a
where
	mappend :: !(Sum a) !(Sum a) -> Sum a | +, zero a

instance Semigroup (Product a) | *, one a
where
	mappend :: !(Product a) !(Product a) -> Product a | *, one a

instance Semigroup (First a)
instance Semigroup (Last a)
where
	mappend :: !(Last a) !(Last a) -> Last a


instance Monoid (Dual a) | Monoid a
instance Monoid (Endo .a)
instance Monoid All
instance Monoid Any
instance Monoid (Sum a) | + a & zero a
instance Monoid (Product a) | * a & one a
instance Monoid (First a)
instance Monoid (Last a)

getDual :: !(Dual .a) -> .a

appEndo :: !(Endo .a) -> (.a -> .a)

getAll :: !All -> Bool

getAny :: !Any -> Bool

getSum :: !(Sum a) -> a

getProduct :: !(Product a) -> a

getFirst :: !(First a) -> ?a

getLast :: !(Last a) -> ?a
