implementation module Data.Monoid

import StdEnv
from StdOverloadedList import ++|

instance Semigroup ()
where
	mappend _ _  = ()

instance Semigroup (x, y) | Semigroup x & Semigroup y
where
	mappend :: !(x, y) !(x, y) -> (x, y) | Semigroup x & Semigroup y
	mappend (x1, y1) (x2, y2) = (mappend x1 x2, mappend y1 y2)

instance Semigroup (x, y, z) | Semigroup x & Semigroup y & Semigroup z
where
	mappend :: !(x, y, z) !(x, y, z) -> (x, y, z) | Semigroup x & Semigroup y & Semigroup z
	mappend (x1, y1, z1) (x2, y2, z2) = (mappend x1 x2, mappend y1 y2, mappend z1 z2)

instance Semigroup (x, y, z, a) | Semigroup x & Semigroup y & Semigroup z & Semigroup a
where
	mappend :: !(x, y, z, a) !(x, y, z, a) -> (x, y, z, a) | Semigroup x & Semigroup y & Semigroup z & Semigroup a
	mappend (x1, y1, z1, a1) (x2, y2, z2, a2) = (mappend x1 x2, mappend y1 y2, mappend z1 z2, mappend a1 a2)

instance Semigroup (x, y, z, a, b) | Semigroup x & Semigroup y & Semigroup z & Semigroup a & Semigroup b
where
	mappend :: !(x, y, z, a, b) !(x, y, z, a, b) -> (x, y, z, a, b) | Semigroup x & Semigroup y & Semigroup z & Semigroup a & Semigroup b
	mappend (x1, y1, z1, a1, b1) (x2, y2, z2, a2, b2) = (mappend x1 x2, mappend y1 y2, mappend z1 z2, mappend a1 a2, mappend b1 b2)

instance Semigroup (x, y, z, a, b, c) | Semigroup x & Semigroup y & Semigroup z & Semigroup a & Semigroup b & Semigroup c
where
	mappend :: !(x, y, z, a, b, c) !(x, y, z, a, b, c) -> (x, y, z, a, b, c) | Semigroup x & Semigroup y & Semigroup z & Semigroup a & Semigroup b & Semigroup c
	mappend (x1, y1, z1, a1, b1, c1) (x2, y2, z2, a2, b2, c2) = (mappend x1 x2, mappend y1 y2, mappend z1 z2, mappend a1 a2, mappend b1 b2, mappend c1 c2)

instance Semigroup [a]
where
	mappend xs ys  = xs ++ ys

instance Semigroup [!a]
where
	mappend xs ys  = xs ++| ys

instance Semigroup [a!]
where
	mappend xs ys = xs ++| ys

instance Semigroup [!a!]
where
	mappend xs ys = xs ++| ys

instance Semigroup (? a) | Semigroup a
where
	mappend :: !(? a) !(? a) -> ? a | Semigroup a
	mappend ?None      m          = m
	mappend m          ?None      = m
	mappend (?Just m1) (?Just m2) = ?Just (mappend m1 m2)
	mappend _          _          = abort "impossible case in mappend_Maybe\n"

instance Semigroup (?^ a) | Semigroup a
where
	mappend :: !(?^ a) !(?^ a) -> ?^ a | Semigroup a
	mappend ?^None      m           = m
	mappend m           ?^None      = m
	mappend (?^Just m1) (?^Just m2) = ?^Just (mappend m1 m2)
	mappend _           _           = abort "impossible case in mappend_Maybe\n"

instance Semigroup (a -> b) | Semigroup b
where
	mappend f g = \x -> mappend (f x) (g x)

instance Monoid ()
where
	mempty = ()

instance Monoid (x, y) | Monoid x & Monoid y
where
	mempty = (mempty, mempty)

instance Monoid (x, y, z) | Monoid x & Monoid y & Monoid z
where
	mempty = (mempty, mempty, mempty)

instance Monoid (x, y, z, a) | Monoid x & Monoid y & Monoid z & Monoid a
where
	mempty = (mempty, mempty, mempty, mempty)

instance Monoid (x, y, z, a, b) | Monoid x & Monoid y & Monoid z & Monoid a & Monoid b
where
	mempty = (mempty, mempty, mempty, mempty, mempty)

instance Monoid (x, y, z, a, b, c) | Monoid x & Monoid y & Monoid z & Monoid a & Monoid b & Monoid c
where
	mempty = (mempty, mempty, mempty, mempty, mempty, mempty)

instance Monoid [a]
where
	mempty = []

instance Monoid [!a]
where
	mempty = [! ]

instance Monoid [a!]
where
	mempty = [ !]

instance Monoid [!a!]
where
	mempty = [!!]

instance Monoid (? a)
where
	mempty = ?None

instance Monoid (?^ a)
where
	mempty = ?^None

instance Monoid (a -> b) | Monoid b
where
	mempty = \_ -> mempty

instance Semigroup (Dual a) | Semigroup a
where
	mappend :: !(Dual a) !(Dual a) -> Dual a | Semigroup a
	mappend (Dual x) (Dual y) = Dual (mappend y x)

instance Monoid (Dual a) | Monoid a
where
	mempty = Dual mempty

instance Semigroup (Endo .a)
where
	mappend :: !(Endo .a) !(Endo .a) -> Endo .a
	mappend (Endo f) (Endo g) = Endo (f o g)

instance Monoid (Endo .a)
where
	mempty = Endo id

instance Semigroup All
where
	mappend :: !All !All -> All
	mappend (All x) (All y) = All (x && y)

instance Monoid All
where
	mempty = All True

instance Semigroup Any
where
	mappend :: !Any !Any -> Any
	mappend (Any x) (Any y) = Any (x || y)

instance Monoid Any
where
	mempty = Any False

instance Semigroup (Sum a) | +, zero a
where
	mappend :: !(Sum a) !(Sum a) -> Sum a | +, zero a
	mappend (Sum x) (Sum y) = Sum (x + y)

instance Monoid (Sum a) | + a & zero a
where
	mempty = Sum zero

instance Semigroup (Product a) | *, one a
where
	mappend :: !(Product a) !(Product a) -> Product a | *, one a
	mappend (Product x) (Product y) = Product (x * y)

instance Monoid (Product a) | * a & one a
where
	mempty = Product one

instance Semigroup (First a)
where
	mappend r=:(First (?Just _)) _ = r
	mappend (First ?None) r = r

instance Monoid (First a)
where
	mempty = First ?None

instance Semigroup (Last a)
where
	mappend :: !(Last a) !(Last a) -> Last a
	mappend _ r=:(Last (?Just _)) = r
	mappend r (Last ?None) = r

instance Monoid (Last a)
where
	mempty = Last ?None

getDual :: !(Dual .a) -> .a
getDual (Dual x) = x

appEndo :: !(Endo .a) -> (.a -> .a)
appEndo (Endo f) = f

getAll :: !All -> Bool
getAll (All b) = b

getAny :: !Any -> Bool
getAny (Any b) = b

getSum :: !(Sum a) -> a
getSum (Sum x) = x

getProduct :: !(Product a) -> a
getProduct (Product x) = x

getFirst :: !(First a) -> ?a
getFirst (First x) = x

getLast :: !(Last a) -> ?a
getLast (Last x) = x
